@@ -1,8 +1,9 @@
-__version__ = "3.1.0"
+__version__ = "3.1.7"
 __description__ = "One-stop solution for HTTP(S) testing."
 
 # import firstly for monkey patch if needed
 from httprunner.ext.locust import main_locusts
+from httprunner.parser import parse_parameters as Parameters
 from httprunner.runner import HttpRunner
 from httprunner.testcase import Config, Step, RunRequest, RunTestCase
 
@@ -14,4 +15,5 @@ __all__ = [
     "Step",
     "RunRequest",
     "RunTestCase",
+    "Parameters",
 ]
