@@ -13,8 +13,9 @@ from httprunner.exceptions import ParamsError
 def gen_random_string(str_len):
     """ generate random string with specified length
     """
-    return ''.join(
-        random.choice(string.ascii_letters + string.digits) for _ in range(str_len))
+    return "".join(
+        random.choice(string.ascii_letters + string.digits) for _ in range(str_len)
+    )
 
 
 def get_timestamp(str_len=13):
@@ -36,4 +37,3 @@ def sleep(n_secs):
     """ sleep n seconds
     """
     time.sleep(n_secs)
-
