@@ -182,6 +182,10 @@ def _ensure_step_attachment(step: Dict) -> Dict:
         test_dict["export"] = step["export"]
 
     if "validate" in step:
+        if not isinstance(step["validate"], List):
+            raise exceptions.TestCaseFormatError(
+                f'Invalid teststep validate: {step["validate"]}'
+            )
         test_dict["validate"] = _convert_validators(step["validate"])
 
     if "validate_script" in step:
@@ -200,8 +204,13 @@ def ensure_testcase_v3_api(api_content: Dict) -> Dict:
 
     teststep = _sort_step_by_custom_order(teststep)
 
+    config = {"name": api_content["name"]}
+    extract_variable_names: List = list(teststep.get("extract", {}).keys())
+    if extract_variable_names:
+        config["export"] = extract_variable_names
+
     return {
-        "config": {"name": api_content["name"]},
+        "config": config,
         "teststeps": [teststep],
     }
 
@@ -333,7 +342,7 @@ def session_fixture(request):
         testcase_summary_json["records"] = testcase_summary_json.pop("step_datas")
         summary["details"].append(testcase_summary_json)
 
-    summary_path = "{{SUMMARY_PATH_PLACEHOLDER}}"
+    summary_path = r"{{SUMMARY_PATH_PLACEHOLDER}}"
     summary_dir = os.path.dirname(summary_path)
     os.makedirs(summary_dir, exist_ok=True)
 
