@@ -2,12 +2,13 @@ from typing import Dict, Text, Any, NoReturn
 
 import jmespath
 import requests
+from jmespath.exceptions import JMESPathError
 from loguru import logger
 
 from httprunner import exceptions
 from httprunner.exceptions import ValidationFailure, ParamsError
-from httprunner.parser import parse_data, parse_string_value, get_mapping_function
 from httprunner.models import VariablesMapping, Validators, FunctionsMapping
+from httprunner.parser import parse_data, parse_string_value, get_mapping_function
 
 
 def get_uniform_comparator(comparator: Text):
@@ -80,6 +81,7 @@ def uniform_validator(validator):
         # format1
         check_item = validator["check"]
         expect_value = validator["expect"]
+        message = validator.get("message", "")
         comparator = validator.get("comparator", "eq")
 
     elif len(validator) == 1:
@@ -87,10 +89,16 @@ def uniform_validator(validator):
         comparator = list(validator.keys())[0]
         compare_values = validator[comparator]
 
-        if not isinstance(compare_values, list) or len(compare_values) != 2:
+        if not isinstance(compare_values, list) or len(compare_values) not in [2, 3]:
             raise ParamsError(f"invalid validator: {validator}")
 
-        check_item, expect_value = compare_values
+        check_item = compare_values[0]
+        expect_value = compare_values[1]
+        if len(compare_values) == 3:
+            message = compare_values[2]
+        else:
+            # len(compare_values) == 2
+            message = ""
 
     else:
         raise ParamsError(f"invalid validator: {validator}")
@@ -98,7 +106,12 @@ def uniform_validator(validator):
     # uniform comparator, e.g. lt => less_than, eq => equals
     assert_method = get_uniform_comparator(comparator)
 
-    return {"check": check_item, "expect": expect_value, "assert": assert_method}
+    return {
+        "check": check_item,
+        "expect": expect_value,
+        "assert": assert_method,
+        "message": message,
+    }
 
 
 class ResponseObject(object):
@@ -131,14 +144,28 @@ class ResponseObject(object):
         self.__dict__[key] = value
         return value
 
-    @property
-    def resp_obj_meta(self):
-        return {
+    def _search_jmespath(self, expr: Text) -> Any:
+        resp_obj_meta = {
             "status_code": self.status_code,
             "headers": self.headers,
             "cookies": self.cookies,
             "body": self.body,
         }
+        if not expr.startswith(tuple(resp_obj_meta.keys())):
+            return expr
+        
+        try:
+            check_value = jmespath.search(expr, resp_obj_meta)
+        except JMESPathError as ex:
+            logger.error(
+                f"failed to search with jmespath\n"
+                f"expression: {expr}\n"
+                f"data: {resp_obj_meta}\n"
+                f"exception: {ex}"
+            )
+            raise
+
+        return check_value
 
     def extract(self, extractors: Dict[Text, Text]) -> Dict[Text, Any]:
         if not extractors:
@@ -146,7 +173,7 @@ class ResponseObject(object):
 
         extract_mapping = {}
         for key, field in extractors.items():
-            field_value = jmespath.search(field, self.resp_obj_meta)
+            field_value = self._search_jmespath(field)
             extract_mapping[key] = field_value
 
         logger.info(f"extract mapping: {extract_mapping}")
@@ -159,6 +186,9 @@ class ResponseObject(object):
         functions_mapping: FunctionsMapping = None,
     ) -> NoReturn:
 
+        variables_mapping = variables_mapping or {}
+        functions_mapping = functions_mapping or {}
+
         self.validation_results = {}
         if not validators:
             return
@@ -177,12 +207,16 @@ class ResponseObject(object):
             check_item = u_validator["check"]
             if "$" in check_item:
                 # check_item is variable or function
-                check_value = parse_data(
+                check_item = parse_data(
                     check_item, variables_mapping, functions_mapping
                 )
-                check_value = parse_string_value(check_value)
+                check_item = parse_string_value(check_item)
+
+            if check_item and isinstance(check_item, Text):
+                check_value = self._search_jmespath(check_item)
             else:
-                check_value = jmespath.search(check_item, self.resp_obj_meta)
+                # variable or function evaluation result is "" or not text
+                check_value = check_item
 
             # comparator
             assert_method = u_validator["assert"]
@@ -193,6 +227,11 @@ class ResponseObject(object):
             # parse expected value with config/teststep/extracted variables
             expect_value = parse_data(expect_item, variables_mapping, functions_mapping)
 
+            # message
+            message = u_validator["message"]
+            # parse message with config/teststep/extracted variables
+            message = parse_data(message, variables_mapping, functions_mapping)
+
             validate_msg = f"assert {check_item} {assert_method} {expect_value}({type(expect_value).__name__})"
 
             validator_dict = {
@@ -201,14 +240,15 @@ class ResponseObject(object):
                 "check_value": check_value,
                 "expect": expect_item,
                 "expect_value": expect_value,
+                "message": message,
             }
 
             try:
-                assert_func(check_value, expect_value)
+                assert_func(check_value, expect_value, message)
                 validate_msg += "\t==> pass"
                 logger.info(validate_msg)
                 validator_dict["check_result"] = "pass"
-            except AssertionError:
+            except AssertionError as ex:
                 validate_pass = False
                 validator_dict["check_result"] = "fail"
                 validate_msg += "\t==> fail"
@@ -219,6 +259,10 @@ class ResponseObject(object):
                     f"assert_method: {assert_method}\n"
                     f"expect_value: {expect_value}({type(expect_value).__name__})"
                 )
+                message = str(ex)
+                if message:
+                    validate_msg += f"\nmessage: {message}"
+
                 logger.error(validate_msg)
                 failures.append(validate_msg)
 
