@@ -21,7 +21,7 @@ from httprunner.loader import load_project_meta, load_testcase_file
 from httprunner.parser import build_url, parse_data, parse_variables_mapping
 from httprunner.response import ResponseObject
 from httprunner.testcase import Config, Step
-from httprunner.utils import override_config_variables
+from httprunner.utils import merge_variables
 from httprunner.models import (
     TConfig,
     TStep,
@@ -89,7 +89,7 @@ class HttpRunner(object):
         return self
 
     def __call_hooks(
-        self, hooks: Hooks, step_variables: VariablesMapping, hook_type: Text,
+        self, hooks: Hooks, step_variables: VariablesMapping, hook_msg: Text,
     ) -> NoReturn:
         """ call hook actions.
 
@@ -106,10 +106,10 @@ class HttpRunner(object):
                 request: parsed request dict
                 response: ResponseObject for current response
 
-            hook_type: setup/teardown
+            hook_msg: setup/teardown request/testcase
 
         """
-        logger.debug(f"call {hook_type} hook actions.")
+        logger.info(f"call hook actions: {hook_msg}")
 
         if not isinstance(hooks, List):
             logger.error(f"Invalid hooks format: {hooks}")
@@ -153,7 +153,7 @@ class HttpRunner(object):
 
         # setup hooks
         if step.setup_hooks:
-            self.__call_hooks(step.setup_hooks, step.variables, "setup")
+            self.__call_hooks(step.setup_hooks, step.variables, "setup request")
 
         # prepare arguments
         method = parsed_request_dict.pop("method")
@@ -169,7 +169,7 @@ class HttpRunner(object):
 
         # teardown hooks
         if step.teardown_hooks:
-            self.__call_hooks(step.teardown_hooks, step.variables, "teardown")
+            self.__call_hooks(step.teardown_hooks, step.variables, "teardown request")
 
         def log_req_resp_details():
             err_msg = "\n{} DETAILED REQUEST & RESPONSE {}\n".format("*" * 32, "*" * 32)
@@ -236,6 +236,10 @@ class HttpRunner(object):
         step_variables = step.variables
         step_export = step.export
 
+        # setup hooks
+        if step.setup_hooks:
+            self.__call_hooks(step.setup_hooks, step_variables, "setup testcase")
+
         if hasattr(step.testcase, "config") and hasattr(step.testcase, "teststeps"):
             testcase_cls = step.testcase
             case_result = (
@@ -269,6 +273,10 @@ class HttpRunner(object):
                 f"Invalid teststep referenced testcase: {step.dict()}"
             )
 
+        # teardown hooks
+        if step.teardown_hooks:
+            self.__call_hooks(step.teardown_hooks, step.variables, "teardown testcase")
+
         step_data.data = case_result.get_step_datas()  # list of step data
         step_data.export_vars = case_result.get_export_variables()
         step_data.success = case_result.success
@@ -327,17 +335,16 @@ class HttpRunner(object):
         self.__start_at = time.time()
         self.__step_datas: List[StepData] = []
         self.__session = self.__session or HttpSession()
-        self.__session_variables = {}
+        # save extracted variables of teststeps
+        extracted_variables: VariablesMapping = {}
 
         # run teststeps
         for step in self.__teststeps:
             # override variables
-            # session variables (extracted from pre step) > step variables
-            step.variables.update(self.__session_variables)
+            # step variables > extracted variables from previous steps
+            step.variables = merge_variables(step.variables, extracted_variables)
             # step variables > testcase config variables
-            step.variables = override_config_variables(
-                step.variables, self.__config.variables
-            )
+            step.variables = merge_variables(step.variables, self.__config.variables)
 
             # parse variables
             step.variables = parse_variables_mapping(
@@ -352,8 +359,9 @@ class HttpRunner(object):
                 extract_mapping = self.__run_step(step)
 
             # save extracted variables to session variables
-            self.__session_variables.update(extract_mapping)
+            extracted_variables.update(extract_mapping)
 
+        self.__session_variables.update(extracted_variables)
         self.__duration = time.time() - self.__start_at
         return self
 
@@ -413,7 +421,7 @@ class HttpRunner(object):
             step_datas=self.__step_datas,
         )
 
-    def test_start(self) -> "HttpRunner":
+    def test_start(self, param: Dict = None) -> "HttpRunner":
         """main entrance, discovered by pytest"""
         self.__init_tests__()
         self.__project_meta = self.__project_meta or load_project_meta(
@@ -427,6 +435,8 @@ class HttpRunner(object):
 
         # parse config name
         config_variables = self.__config.variables
+        if param:
+            config_variables.update(param)
         config_variables.update(self.__session_variables)
         self.__config.name = parse_data(
             self.__config.name, config_variables, self.__project_meta.functions
