@@ -1,17 +1,20 @@
 import collections
 import copy
+import itertools
 import json
 import os.path
 import platform
 import uuid
-from typing import Dict, List, Any
+from multiprocessing import Queue
+from typing import Any, Dict, List, Text
 
+import requests
 import sentry_sdk
-from httprunner import __version__
-from httprunner import exceptions
-from httprunner.models import VariablesMapping
 from loguru import logger
 
+from httprunner import __version__, exceptions
+from httprunner.models import VariablesMapping
+
 
 def init_sentry_sdk():
     sentry_sdk.init(
@@ -22,6 +25,48 @@ def init_sentry_sdk():
         scope.set_user({"id": uuid.getnode()})
 
 
+class GAClient(object):
+
+    version = '1'   # GA API Version
+    report_url = 'https://www.google-analytics.com/collect'
+    report_debug_url = 'https://www.google-analytics.com/debug/collect'   # used for debug
+
+    def __init__(self, tracking_id: Text):
+        self.http_client = requests.Session()
+        self.label = str(__version__)
+        self.common_params = {
+            'v': self.version,
+            'tid': tracking_id,    # Tracking ID / Property ID, XX-XXXXXXX-X
+            'cid': uuid.getnode(),      # Anonymous Client ID
+            'ua': f'HttpRunner/{__version__}',
+        }
+
+    def track_event(self, category: Text, action: Text, value: int = 0):
+        data = {
+            't': 'event',       # Event hit type = event
+            'ec': category,     # Required. Event Category.
+            'ea': action,       # Required. Event Action.
+            'el': self.label,   # Optional. Event label, used as version.
+            'ev': value,        # Optional. Event value, must be non-negative integer
+        }
+        data.update(self.common_params)
+        self.http_client.post(self.report_url, data=data)
+
+    def track_user_timing(self, category: Text, variable: Text, duration: int):
+        data = {
+            't': 'timing',      # Event hit type = timing
+            'utc': category,    # Required. user timing category. e.g. jsonLoader
+            'utv': variable,    # Required. timing variable. e.g. load
+            'utt': duration,    # Required. time took duration.
+            'utl': self.label,  # Optional. user timing label, used as version.
+        }
+        data.update(self.common_params)
+        self.http_client.post(self.report_url, data=data)
+
+
+ga_client = GAClient("UA-114587036-1")
+
+
 def set_os_environ(variables_mapping):
     """ set variables mapping to os.environ
     """
@@ -190,15 +235,13 @@ class ExtendJSONEncoder(json.JSONEncoder):
             return repr(obj)
 
 
-def override_config_variables(
-    step_variables: VariablesMapping, config_variables: VariablesMapping
+def merge_variables(
+    variables: VariablesMapping, variables_to_be_overridden: VariablesMapping
 ) -> VariablesMapping:
-    """ override variables:
-            testcase step variables > testcase config variables
-            testsuite testcase variables > testsuite config variables
+    """ merge two variables mapping, the first variables have higher priority
     """
     step_new_variables = {}
-    for key, value in step_variables.items():
+    for key, value in variables.items():
         if f"${key}" == value or "${" + key + "}" == value:
             # e.g. {"base_url": "$base_url"}
             # or {"base_url": "${base_url}"}
@@ -206,6 +249,56 @@ def override_config_variables(
 
         step_new_variables[key] = value
 
-    variables = copy.deepcopy(config_variables)
-    variables.update(step_new_variables)
-    return variables
+    merged_variables = copy.copy(variables_to_be_overridden)
+    merged_variables.update(step_new_variables)
+    return merged_variables
+
+
+def is_support_multiprocessing() -> bool:
+    try:
+        Queue()
+        return True
+    except (ImportError, OSError):
+        # system that does not support semaphores(dependency of multiprocessing), like Android termux
+        return False
+
+
+def gen_cartesian_product(*args: List[Dict]) -> List[Dict]:
+    """ generate cartesian product for lists
+
+    Args:
+        args (list of list): lists to be generated with cartesian product
+
+    Returns:
+        list: cartesian product in list
+
+    Examples:
+
+        >>> arg1 = [{"a": 1}, {"a": 2}]
+        >>> arg2 = [{"x": 111, "y": 112}, {"x": 121, "y": 122}]
+        >>> args = [arg1, arg2]
+        >>> gen_cartesian_product(*args)
+        >>> # same as below
+        >>> gen_cartesian_product(arg1, arg2)
+            [
+                {'a': 1, 'x': 111, 'y': 112},
+                {'a': 1, 'x': 121, 'y': 122},
+                {'a': 2, 'x': 111, 'y': 112},
+                {'a': 2, 'x': 121, 'y': 122}
+            ]
+
+    """
+    if not args:
+        return []
+    elif len(args) == 1:
+        return args[0]
+
+    product_list = []
+    for product_item_tuple in itertools.product(*args):
+        product_item_dict = {}
+        for item in product_item_tuple:
+            product_item_dict.update(item)
+
+        product_list.append(product_item_dict)
+
+    return product_list
